if __name__ == "__main__":
    from netflix_open_content_helper.cli import app

    # Check if the script is being run directly
    if __name__ == "__main__":
        # Call the main function of the Typer app
        app()
